//	Standard includes
#include <string>
#include <iostream>

//	Numerical ice tank includes
#include "SimulationInterface.h"

//	Namespace using

//	Defines and macros

//	Constants and enums

//	Local classes
//		Log message listener
class MessageListener: public IceTankSimulation::LogMessageListener
{
	virtual void onLogMessage(const std::string& message)
	{
		std::cout<<message;
	}
};

//	Local functions
void requestToPressEnter(void)
{
	std::cout<<"Press enter to exit"<<std::endl;
	std::getchar();
}

void logStepResult(const IceTankSimulation::SimulationStepResult& stepResult)
{
	std::cout<<"Ship position: "<< stepResult.shipPosition[0] << " " << stepResult.shipPosition[1] << " " << stepResult.shipPosition[2]<<std::endl;
	std::cout<<"Ship orientation: "<< stepResult.shipOrientation[0] << " " << stepResult.shipOrientation[1] << " " << stepResult.shipOrientation[2]<<std::endl;
	
	std::cout<<"Ship linear velocity: "<< stepResult.shipLinearVelocity[0] << " " << stepResult.shipLinearVelocity[1] << " " << stepResult.shipLinearVelocity[2]<<std::endl;
	std::cout<<"Ship angular velocity: "<< stepResult.shipAngularVelocity[0] << " " << stepResult.shipAngularVelocity[1] << " " << stepResult.shipAngularVelocity[2]<<std::endl;
	
	std::cout<<"Ship linear acceleration: "<< stepResult.shipLinearAcceleration[0] << " " << stepResult.shipLinearAcceleration[1] << " " << stepResult.shipLinearAcceleration[2]<<std::endl;
	std::cout<<"Ship angular acceleration: "<< stepResult.shipAngularAcceleration[0] << " " << stepResult.shipAngularAcceleration[1] << " " << stepResult.shipAngularAcceleration[2]<<std::endl;
	std::cout<<std::endl;
}

void shutdownSimulation(void)
{
	IceTankSimulation::SimulationInterface::releaseInstance();
	IceTankSimulation::SimulationInterface::shutdown();
}

int main(int argc, const char *argv[])
{
	if (argc != 2)
	{
		std::cout<<"Usage: NonVisualSimulation test_file_name"<<std::endl;
		requestToPressEnter();
		return 0;
	}

	const std::string testFilePath = argv[1];
	MessageListener messageListener;
	try
	{
		IceTankSimulation::SimulationInterface& simInterface = IceTankSimulation::SimulationInterface::createInstance();
		
		simInterface.addLogMessageListener(&messageListener);
		simInterface.createTest(testFilePath);
		while (!simInterface.isSimulationCompleted())
		{
			IceTankSimulation::SimulationStepResult stepResult = simInterface.step(IceTankSimulation::DPInput(50.0, 0.0, 1.0));
			logStepResult(stepResult);
		}
		shutdownSimulation();
	}
	catch (std::exception& exception)
	{
		shutdownSimulation();
		std::cout<<std::endl<<"An ERROR occured while running the test '"<<testFilePath<<"': "<<std::endl<<exception.what()<<std::endl<<std::endl;
		requestToPressEnter();
	}
	catch (...)
	{
		shutdownSimulation();
		std::cout<<std::endl<<"An unknown ERROR occured while running the test '"<<testFilePath<<"'"<<std::endl<<std::endl;
		requestToPressEnter();
		return 1;
	}
}

