/*
    VisualizationInterface header
*/

//  Multiple include guard start
#ifndef ICETANKVISUALIZATIONLIB_VISUALIZATIONINTERFACE_H_INCLUDED__
#define ICETANKVISUALIZATIONLIB_VISUALIZATIONINTERFACE_H_INCLUDED__

//  Standard includes

//  Simulation includes
#include "SimulationInterface.h"

//		DYPIC_Visual::VisualizationManager forward declaration
namespace DYPIC_Visual
{
	class VisualizationManager;
}

//  Enter namespace
namespace IceTankVisualization
{
//  Constants and enums

//  Global classes
class VisualizationInterface
{
public:
//	Public methods
/*
		Creates visualization
		Params: simulation interface
		Return: none
*/
	void createVisualization(IceTankSimulation::SimulationInterface& simulationInterface, const std::string& filePath);

/*
		Check if the visualization is already created
		Params: none
		Return: result of the check
*/
	bool isVisualizationCreated(void) const;

/*
		Updates visualization window
		Params: none
		Return: none
*/
	void updateVisualizationWindow(void);

/*
		Checks if the visualization window is closed
		Params: none
		Return: result of the check
*/
	bool isVisualizationWindowClosed(void) const;

//	Public static methods
/*
		Instance creator
		Params: none
		Return: created instance of VisualizationInterface
		Note: VisualizationInterface is a singleton, multiple calls of createInstance() are not allowed.
			However, you can call createInstance()-releaseInstance() multiple times.
*/
	static VisualizationInterface& createInstance();

/*
		Current instance existence checker
		Params: none
		Return: existence state of the current instance
*/
	static bool isInstanceCreated(void);

/*
		Current instance getter
		Params: none
		Return: current instance of VisualizationInterface
*/
	static VisualizationInterface& getInstance(void);

/*
		Releaser
		Params: none
		Return: none
		Note: this method must be called in order to release the resources used by the current instance of the VisualizationInterface
*/
	static void releaseInstance(void);

private:
//	Private constructors
/*
        VisualizationInterface default constructor
        Params: none
*/
	VisualizationInterface(void);

/*
        VisualizationInterface copy constructor
        Params: none
		Note: this constructor is intentionally declared but not defined
*/
	VisualizationInterface(const VisualizationInterface& source);

//	Private methods
/*
        VisualizationInterface assigment operator
        Params: none
		Note: this method is intentionally declared but not defined
*/
	VisualizationInterface& operator=(const VisualizationInterface& source);

/*
        Releases resources used by the VisualizationInterface
        Params: none
		Return: none
*/
	void release(void);

//	Members
//		Implementation of the visualization interface
	DYPIC_Visual::VisualizationManager* mImplementation;

//	Private static members
//		Instance of the visualization interface
	static VisualizationInterface* sCurrentInstance;
};

//  Global functions

//  Exit namespace
}

//  Multiple include guard end
#endif
